@@ -16,14 +16,13 @@
 // with Gnome Weather; if not, write to the Free Software Foundation,
 // Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-const Atk = imports.gi.Atk;
-const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const GLib = imports.gi.GLib;
 const Gnome = imports.gi.GnomeDesktop;
 const Lang = imports.lang;
 
 const Forecast = imports.app.forecast;
+const WForecast = imports.app.weeklyForecast;
 const Params = imports.misc.params;
 const Util = imports.misc.util;
 
@@ -39,49 +38,115 @@ const WeatherWidget = new Lang.Class({
         this.parent(params);
 
         this._currentStyle = null;
+        this._info = null;
 
         let builder = new Gtk.Builder();
         builder.add_from_resource('/org/gnome/Weather/Application/city.ui');
 
         let outerBox = builder.get_object('outer-box');
         this._contentFrame = builder.get_object('content-frame');
-        let outerGrid = builder.get_object('outer-grid');
+        this._outerGrid = builder.get_object('outer-grid');
+        this._forecastGrid = builder.get_object('forecast-grid');
+        this._wForecastFrame = builder.get_object('weekly-forecast-frame');
+        let forecastScrollingWindow = builder.get_object('forecast-scrolled-window');
         this._icon = builder.get_object('conditions-image');
         this._temperature = builder.get_object('temperature-label');
         this._conditions = builder.get_object('conditions-label');
-        this._revealButton = builder.get_object('reveal-button');
-        this._revealer = builder.get_object('revealer');
-        this._time = builder.get_object('time-label');
+        this.timeLabel = builder.get_object('time-label');
+        this.timeGrid = builder.get_object('time-grid');
+        this._dayStack = builder.get_object('day-stack');
+        this._leftButton = builder.get_object('left-button');
+        this._rightButton = builder.get_object('right-button');
 
-        this._forecasts = new Forecast.ForecastBox({ hexpand: true });
-        outerGrid.attach(this._forecasts, 0, 2, 1, 1);
+        this._forecasts = new Forecast.ForecastBox({ hexpand: false });
+        this._forecastGrid.attach(this._forecasts, 0, 0, 1, 1);
 
-        this._today = new Forecast.TodaySidebar({ vexpand: true,
-                                                  name: 'today-sidebar' });
-        this._revealer.child = this._today;
+        this._weeklyForecasts = new WForecast.WeeklyForecastFrame();
+        this._outerGrid.attach(this._weeklyForecasts, 1, 0, 1, 2);
 
-        this._revealButton.connect('clicked', Lang.bind(this, function() {
-            this._revealer.reveal_child = !this._revealer.reveal_child;
-            this._syncRevealButton();
+        this._hscrollbar = forecastScrollingWindow.get_hscrollbar();
+        this._hscrollbar.set_opacity(0.0);
+
+        this._hadjustment = forecastScrollingWindow.get_hadjustment();
+
+        this._hadjustment.connect('changed', Lang.bind(this, function() {
+            if ((this._hadjustment.get_upper() - this._hadjustment.get_lower()) == this._hadjustment.page_size) {
+                this._leftButton.set_sensitive(false);
+                this._rightButton.set_sensitive(false);
+            } else if (this._hadjustment.value == this._hadjustment.get_lower()){
+                this._leftButton.set_sensitive(false);
+                this._rightButton.set_sensitive(true);
+            } else if (this._hadjustment.value >= (this._hadjustment.get_upper() - this._hadjustment.page_size)){
+                this._leftButton.set_sensitive(true);
+                this._rightButton.set_sensitive(false);
+            } else {
+                this._leftButton.set_sensitive(true);
+                this._rightButton.set_sensitive(true);
+            }
+        }));
+
+        this._dayStack.connect('notify::visible-child', Lang.bind(this, function() {
+            this.clear();
+            if (this._info) {
+                let forecasts = this._info.get_forecast_list();
+                this._forecasts.update(forecasts, this._dayStack.get_visible_child_name());
+                this._hadjustment.value = this._hadjustment.get_lower();
+                this._forecasts.show();
+            }
+        }));
+
+        this._leftButton.connect('clicked', Lang.bind(this, function() {
+            this._target = this._hadjustment.value - this._hadjustment.page_size;
+            if (this._target <= this._hadjustment.get_lower()) {
+                this._leftButton.set_sensitive(false);
+                this._rightButton.set_sensitive(true);
+            } else
+                this._rightButton.set_sensitive(true);
+
+            this._start = new Date().getTime();
+            this._end = this._start + 328;
+            this._tickId = this._forecastGrid.add_tick_callback(Lang.bind(this, this._animate));
+        }));
+
+        this._rightButton.connect('clicked', Lang.bind(this, function() {
+            this._target = this._hadjustment.value + this._hadjustment.page_size;
+            if (this._target >= this._hadjustment.get_upper() - this._hadjustment.page_size) {
+                this._rightButton.set_sensitive(false);
+                this._leftButton.set_sensitive(true);
+            } else
+                this._leftButton.set_sensitive(true);
+
+            this._start = new Date().getTime();
+            this._end = this._start + 328;
+            this._tickId = this._forecastGrid.add_tick_callback(Lang.bind(this, this._animate));
         }));
-        this._syncRevealButton();
 
         this.add(outerBox);
     },
 
-    _syncRevealButton: function() {
-        if (this._revealer.reveal_child) {
-            this._revealButton.get_child().icon_name = 'go-next-symbolic';
-            this._revealButton.get_accessible().ref_state_set().add_state(Atk.StateType.CHECKED);
+    _animate: function() {
+        let value = this._hadjustment.value;
+        let t = 1.0;
+        let now = new Date().getTime();
+        if (now < this._end) {
+            t = (now - this._start) / 700;
+            t = this._easeOutCubic (t);
+            this._hadjustment.value = value + t * (this._target - value);
+            return true;
         } else {
-            this._revealButton.get_child().icon_name = 'go-previous-symbolic';
-            this._revealButton.get_accessible().ref_state_set().remove_state(Atk.StateType.CHECKED);
+            this._hadjustment.value = value + t * (this._target - value);
+            this._forecastGrid.remove_tick_callback(this._tickId);
+            return false;
         }
     },
 
+    _easeOutCubic: function(value) {
+        let temp = value - 1;
+        return temp * temp * temp + 1;
+    },
+
     clear: function() {
         this._forecasts.clear();
-        this._today.clear();
     },
 
     _get_style_class: function(info) {
@@ -91,6 +156,8 @@ const WeatherWidget = new Lang.Class({
     },
 
     update: function(info) {
+        this._info = info;
+
         this._conditions.label = Util.getWeatherConditions(info);
         this._temperature.label = info.get_temp_summary();
 
@@ -102,17 +169,14 @@ const WeatherWidget = new Lang.Class({
         context.add_class(this._currentStyle);
 
         let forecasts = info.get_forecast_list();
-        if (forecasts.length > 0) {
-            this._forecasts.update(forecasts);
-            this._forecasts.show();
+        this._forecasts.update(forecasts, this._dayStack.get_visible_child_name());
+        this._forecasts.show();
 
-            this._today.update(info);
-            this._revealButton.show();
-            this._revealer.show();
+        if (forecasts.length == 0) {
+            this._weeklyForecasts.hide();
         } else {
-            this._forecasts.hide();
-            this._revealButton.hide();
-            this._revealer.hide();
+            this._weeklyForecasts.show();
+            this._weeklyForecasts.update(forecasts);
         }
     }
 });
@@ -136,13 +200,16 @@ const WeatherView = new Lang.Class({
                                         name: "loading-label" }));
         this.add_named(loadingPage, 'loading');
 
-        this._infoPage = new WeatherWidget();
-        this.add_named(this._infoPage, 'info');
+        this.infoPage = new WeatherWidget();
+        this.add_named(this.infoPage, 'info');
 
         this._info = null;
         this._updateId = 0;
 
         this.connect('destroy', Lang.bind(this, this._onDestroy));
+
+        this._wallClock = new Gnome.WallClock();
+        this._clockHandlerId = null;
     },
 
     get info() {
@@ -154,7 +221,7 @@ const WeatherView = new Lang.Class({
             this._info.disconnect(this._updateId);
             this._updateId = 0;
 
-            this._infoPage.clear();
+            this.infoPage.clear();
         }
 
         this._info = info;
@@ -177,26 +244,25 @@ const WeatherView = new Lang.Class({
     update: function() {
         this.visible_child_name = 'loading';
         this._spinner.start();
-        this._infoPage.clear();
+        this.infoPage.clear();
 
         getApp().model.updateInfo(this._info);
     },
 
     _onUpdate: function(info) {
-        this._infoPage.clear();
-        this._infoPage.update(info);
+        this.infoPage.clear();
+        this.infoPage.update(info);
         this._updateTime();
         this._spinner.stop();
         this.visible_child_name = 'info';
     },
 
     connectClock: function() {
-        this._wallClock = new Gnome.WallClock();
         this._clockHandlerId = this._wallClock.connect('notify::clock', Lang.bind(this, this._updateTime));
     },
 
     _updateTime: function() {
-        this._infoPage._time.label = this._getTime();
+        this.infoPage.timeLabel.label = this._getTime();
     },
 
     _getTime: function() {
@@ -210,6 +276,9 @@ const WeatherView = new Lang.Class({
     },
 
     disconnectClock: function() {
-        this._wallClock.disconnect(this._clockHandlerId);
+        if (this._clockHandlerId) {
+            this._wallClock.disconnect(this._clockHandlerId);
+            this._clockHandlerId = null;
+        }
     }
 });
